@@ -1,18 +1,20 @@
 package Net::Ping::External;
 
-# Author:   Colin McMillen (colinm@cpan.org)
+# Author:   Colin McMillen (colinm AT cpan.org)
 # See also the CREDITS section in the POD below.
 #
-# Copyright (c) 2001 Colin McMillen.  All rights reserved.  This
+# Copyright (c) 2001-2003 Colin McMillen.  All rights reserved.  This
 # program is free software; you may redistribute it and/or modify it
 # under the same terms as Perl itself.
+# Copyright (c) 2006-2007 Alexandr Ciornii
 
-use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
+use strict;
+use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $DEBUG);
 use Carp;
 use Socket qw(inet_ntoa);
 require Exporter;
 
-$VERSION = "0.10";
+$VERSION = "0.12";
 @ISA = qw(Exporter);
 @EXPORT = qw();
 @EXPORT_OK = qw(ping);
@@ -34,19 +36,19 @@ sub ping {
   my %dispatch = 
     (linux    => \&_ping_linux,
      mswin32  => \&_ping_win32,
-     cygwin   => \&_ping_win32,
+     cygwin   => \&_ping_cygwin,
      solaris  => \&_ping_solaris,
      bsdos    => \&_ping_bsdos,
      beos     => \&_ping_beos,
      hpux     => \&_ping_hpux,
      dec_osf  => \&_ping_dec_osf,
      bsd      => \&_ping_bsd,
-     darwin   => \&_ping_unix,
+     darwin   => \&_ping_darwin,
      openbsd  => \&_ping_unix,
      freebsd  => \&_ping_freebsd,
      next     => \&_ping_next,
      unicosmk => \&_ping_unicosmk,
-     netbsd   => \&_ping_unix,
+     netbsd   => \&_ping_netbsd,
      irix     => \&_ping_unix,
      aix      => \&_ping_aix,
     );
@@ -66,7 +68,10 @@ sub _ping_win32 {
   my $command = "ping -l $args{size} -n $args{count} -w $args{timeout} $args{host}";
   print "$command\n" if $DEBUG;
   my $result = `$command`;
-  return 1 if $result =~ /time?\d*ms/;
+  return 1 if $result =~ /time.*ms/;
+  return 1 if $result =~ /TTL/;
+  return 1 if $result =~ /is alive/; # ppt (from CPAN) ping
+#  return 1 if $result !~ /\(100%/; # 100% packages lost
   return 0;
 }
 
@@ -80,6 +85,7 @@ sub _ping_system {
      ) = @_;
   my $devnull = "/dev/null";
   $command .= " 1>$devnull 2>$devnull";
+  print "#$command\n" if $DEBUG;
   my $exit_status = system($command) >> 8;
   return 1 if $exit_status == $success;
   return 0;
@@ -88,6 +94,21 @@ sub _ping_system {
 # Below are all the systems on which _ping_system() has been tested
 # and found OK.
 
+# Mac OS X 10.2 ping does not handle -w timeout now does it return a
+# status code if it fails to ping (unless it cannot resolve the domain 
+# name)
+# Thanks to Peter N. Lewis for this one.
+sub _ping_darwin {
+   my %args = @_;
+   my $command = "ping -s $args{size} -c $args{count} $args{host}";
+   my $devnull = "/dev/null";
+   $command .= " 2>$devnull";
+   print "$command\n" if $DEBUG;
+   my $result = `$command`;
+   return 1 if $result =~ /(\d+) packets received/ && $1 > 0;
+   return 0;
+}
+
 # Assumed OK for DEC OSF
 sub _ping_dec_osf {
   my %args = @_;
@@ -145,18 +166,39 @@ sub _ping_unix {
   return _ping_system($command, 0);
 }
 
+
+sub _locate_ping_netbsd {
+ return '/usr/sbin/ping' if (-x '/usr/sbin/ping');
+ return 'ping';
+}
+
+sub _ping_netbsd {
+  my %args = @_;
+  my $command = _locate_ping_netbsd()." -s $args{size} -c $args{count} -w $args{timeout} $args{host}";
+  return _ping_system($command, 0);
+}
+#-s size -c count -w timeout 
+#http://netbsd.gw.com/cgi-bin/man-cgi?ping++NetBSD-current
+
 # Assumed OK for FreeBSD 3.4
 # -s size option supported -- superuser only... fixme
 sub _ping_bsd {
   my %args = @_;
   my $command = "ping -c $args{count} -q $args{hostname}";
+  return _ping_system($command, 0);
 }
 
 # Debian 2.2 OK, RedHat 6.2 OK
 # -s size option available to superuser... FIXME?
 sub _ping_linux {
   my %args = @_;
-  my $command = "ping -c $args{count} $args{host}";
+  my $command;
+#for next version
+#  if (-e '/etc/redhat-release' || -e '/etc/SuSE-release') {
+#   $command = "ping -c $args{count} -s $args{size} $args{host}";
+#  } else {
+  $command = "ping -c $args{count} $args{host}";
+#  }
   return _ping_system($command, 0);
 }
 
@@ -176,6 +218,15 @@ sub _ping_freebsd {
     return _ping_system($command, 0);
 }
 
+#No timeout
+#Usage:  ping [-dfqrv] host [packetsize [count [preload]]]
+sub _ping_cygwin {
+  my %args = @_;
+  my $command = "ping $args{host} $args{size} $args{count}";
+  return _ping_system($command, 0);
+}
+#Problem is that we may be running windows ping
+
 1;
 
 __END__
@@ -306,7 +357,12 @@ to locate your system's `ping' command.
 
 =item * Win32
 
-Tested OK on Win98. It should work on other Windows systems as well.
+Tested OK on Win98, Win XP. It should work on other Windows systems as well.
+
+=item * Cygwin
+
+Tested OK on Cygwin 1.5.21. Problem is that we may be running windows ping.
+They have different options.
 
 =item * Linux
 
@@ -317,7 +373,7 @@ C<count> are disabled.
 
 =item * BSD
 
-Tested OK on OpenBSD 2.7. Needs testing for FreeBSD, NetBSD, and BSDi.
+Tested OK on OpenBSD 2.7 and 3.0, Netbsd 1.5.3, Freebsd 4.6.2, 5.4. Needs testing for BSDi.
 
 =item * Solaris
 
@@ -361,7 +417,7 @@ Beware Greeks bearing gifts.
 
 =head1 AUTHOR
 
-Colin McMillen (colinm@cpan.org)
+Alexandr Ciornii (alexchorny AT gmail.com), Colin McMillen (colinm AT cpan.org)
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -381,6 +437,11 @@ Marc-Andre Dumas contributed a patch for FreeBSD support.
 Jonathan Stowe fixed a bug in 0.09 that prevented the module from
 running on some systems.
 
+Numerous people sent in a patch to fix a bug in 0.10 that broke ping on Windown systems.
+
+Peter N. Lewis contributed a patch that works correctly on Mac OS X
+10.2 (and hopefully other versions as well).
+
 =head1 SEE ALSO
 
 Net::Ping
